import logging

from PIL import Image

from label_studio_tools.core.utils.params import get_env
from label_studio_tools.core.utils.io import get_local_path

DATA_UNDEFINED_NAME = '$undefined$'

logger = logging.getLogger(__name__)


def get_single_tag_keys(parsed_label_config, control_type, object_type):
    """
    Gets parsed label config, and returns data keys related to the single control tag and the single object tag schema
    (e.g. one "Choices" with one "Text")
    :param parsed_label_config: parsed label config returned by "label_studio.misc.parse_config" function
    :param control_type: control tag str as it written in label config (e.g. 'Choices')
    :param object_type: object tag str as it written in label config (e.g. 'Text')
    :return: 3 string keys and 1 array of string labels: (from_name, to_name, value, labels)
    """
    assert len(parsed_label_config) == 1
    from_name, info = list(parsed_label_config.items())[0]
    assert info['type'] == control_type, 'Label config has control tag "<' + info['type'] + '>" but "<' + control_type + '>" is expected for this model.'  # noqa

    assert len(info['to_name']) == 1
    assert len(info['inputs']) == 1
    assert info['inputs'][0]['type'] == object_type
    to_name = info['to_name'][0]
    value = info['inputs'][0]['value']
    return from_name, to_name, value, info['labels']


def is_skipped(completion):
    if len(completion['annotations']) != 1:
        return False
    completion = completion['annotations'][0]
    return completion.get('skipped', False) or completion.get('was_cancelled', False)


def get_choice(completion):
    return completion['annotations'][0]['result'][0]['value']['choices'][0]


def get_image_local_path(url, image_cache_dir=None, project_dir=None, image_dir=None):
    return get_local_path(url, image_cache_dir, project_dir, get_env('HOSTNAME'), image_dir)


def get_image_size(filepath):
    return Image.open(filepath).size



import traceback as tb
import logging

from flask import request, jsonify, make_response

logger = logging.getLogger(__name__)


# make an answer to client
def answer(status=0, msg='', result=None):
    if status == 0 and not msg and result is None:
        status = -1000
        msg = "nothing happened"

    if status == 200 and not msg:
        msg = 'ok'

    a = {"status": status, "detail": msg}
    a.update({'request': request.args})

    if result is not None:
        a.update({"result": result})

    return make_response(jsonify(a), status)


# make an answer as exception
class AnswerException(Exception):
    def __init__(self, status, msg='', result=None):
        self.status, self.msg, self.result = status, msg, result
        self.answer = answer(status, msg, result)
        Exception.__init__(self, self.answer)


# standard exception treatment for any api function
def exception_handler(f):
    def exception_f(*args, **kwargs):
        try:
            return f(*args, **kwargs)

        except AnswerException as e:
            traceback = tb.format_exc()
            logger.error(traceback)
            if 'traceback' not in e.result:
                e.result['traceback'] = traceback
            if hasattr(exception_f, 'request_id') and not e.result['request_id']:
                e.result['request_id'] = exception_f.request_id

            return answer(e.status, e.msg, e.result)

        except Exception as e:
            traceback = tb.format_exc()
            logger.error(traceback)
            print(traceback)
            body = {'traceback': traceback}
            if hasattr(exception_f, 'request_id'):
                body['request_id'] = exception_f.request_id
            return answer(500, e.__class__.__name__ + ': ' + str(e), body)

    exception_f.__name__ = f.__name__
    return exception_f


from abc import abstractmethod
from .model import LabelStudioMLBase


class LabelStudioMLBaseHelper(LabelStudioMLBase):

    @abstractmethod
    def prepare_tasks(self, tasks, workdir=None, **kwargs):
        pass

    @abstractmethod
    def convert_predictions(self, predictions, **kwargs):
        pass

    @abstractmethod
    def predict2(self, X, y=None, **kwargs):
        pass

    @abstractmethod
    def fit2(self, X, y, **kwargs):
        pass

    def predict(self, tasks, **kwargs):
        X, y = self.prepare_tasks(tasks, **kwargs)
        predictions = self.predict2(X, y, **kwargs)
        result = self.convert_predictions(predictions, **kwargs)
        return result

    def fit(self, completions, workdir=None, **kwargs):
        X, y = self.prepare_tasks(completions, workdir=workdir, **kwargs)
        return self.fit2(X, y, **kwargs)

    def _has_annotation(self, task):
        return 'completions' in task


class LabelStudioMLChoices(LabelStudioMLBaseHelper):

    def __init__(self, **kwargs):
        super(LabelStudioMLChoices, self).__init__(**kwargs)
        assert len(self.parsed_label_config) == 1
        self.from_name, self.info = list(self.parsed_label_config.items())[0]
        assert self.info['type'] == 'Choices'
        assert len(self.info['to_name']) == 1
        assert len(self.info['inputs']) == 1
        self.to_name = self.info['to_name'][0]
        self.value = self.info['inputs'][0]['value']

    def prepare_tasks(self, tasks, workdir=None, **kwargs):
        X, y = [], []
        for task in tasks:
            X.append(task['data'][self.value])
            if self._has_annotation(task):
                choices = task['completions'][0]['result'][0]['value']['choices']
                y.append(choices)
            else:
                y.append(None)
        return X, y

    def convert_predictions(self, predictions, **kwargs):
        list_choices, scores = predictions
        results = []
        for choices, score in zip(list_choices, scores):
            result = [{
                'from_name': self.from_name,
                'to_name': self.to_name,
                'type': 'choices',
                'value': {'choices': choices}
            }]
            results.append({'result': result, 'score': score})
        return results